#!/usr/bin/ruby

$: << "/usr/share/rails/activerecord/lib"

require 'active_record'


# interface for Debian-aware classes
module DebianMethods
	# compare version the Debian way
	def compare_versions(lver, op, rver)
		system("dpkg --compare-versions \"#{lver}\" \"#{op}\" \"#{rver}\"")
		return ($? == 0)
	end
end

# interface for Debian-aware objects
module DebianPackage
	# add class methods
	def self.included(base)
		base.extend DebianMethods
	end

	def <=>(pkg)
		return -1 if self.class.compare_versions(version, "<<", pkg.version)
		return 0 if self.class.compare_versions(version, "=", pkg.version)
		return 1 if self.class.compare_versions(version, ">>", pkg.version)
		raise "Comparing Debian versions failed !"
	end
end


class Suite < ActiveRecord::Base
	set_table_name "suite"

	has_many :binassos, :foreign_key => 'suite'
	has_many :binaries, :source=>"binary_pkg", :through => :binassos
	has_many :srcassos, :foreign_key => 'suite'
	has_many :sources, :source=>"source_pkg", :through => :srcassos
end

class Binasso < ActiveRecord::Base
	set_table_name "bin_associations"

	belongs_to :bin_suite, :class_name => "Suite", :foreign_key => "suite"
	belongs_to :binary_pkg, :class_name => "Binary", :foreign_key => "bin"

	def self.find_all_by_suite_id(id)
		find_all_by_suite(id)
	end
end

class Binary < ActiveRecord::Base
	set_inheritance_column "inheritance_type"   # to avoid using the 'type' field which is used for a common field usage
	include DebianPackage

	# remove associated file manually _after_ this one is done, as the foreign key is on the wrong object
	after_destroy { |record| PkgFile.destroy_all "id = #{record.file}" }

	has_many :binassos, :foreign_key => 'bin', :dependent => :destroy
	has_many :suites, :source => "bin_suite", :through => :binassos
	belongs_to :source_pkg, :class_name => "Source", :foreign_key => "source"
	belongs_to :debfile, :class_name => "PkgFile", :foreign_key => "file"

	def suite_names
		self.suites.collect {|s| s.suite_name}
	end
end

class Srcasso < ActiveRecord::Base
	set_table_name "src_associations"

	belongs_to :src_suite, :class_name => "Suite", :foreign_key => "suite"
	belongs_to :source_pkg, :class_name => "Source", :foreign_key => "source"

	def self.find_all_by_suite_id(id)
		find_all_by_suite(id)
	end
end

class Source < ActiveRecord::Base
	set_table_name "source"
	include DebianPackage

	# remove associated file manually _after_ this one is done, as the foreign key is on the wrong object
	after_destroy { |record| PkgFile.destroy_all "id = #{record.file}" }

	has_many :srcassos, :foreign_key => 'source', :dependent => :destroy
	has_many :suites, :source => "src_suite", :through => :srcassos
	has_many :src_file_asso, :class_name => "SrcFileAsso", :foreign_key => "source", :dependent => :destroy
	has_many :binary_pkgs, :class_name => "Binary", :foreign_key => "source", :dependent => :destroy
	has_many :files, :source=>"src_file", :through => :src_file_asso
	belongs_to :dscfile, :class_name => "PkgFile", :foreign_key => "file"

	def suite_names
		self.suites.collect {|s| s.suite_name}
	end
end

class SrcFileAsso < ActiveRecord::Base
	set_table_name "dsc_files"

	belongs_to :src_pkg, :class_name => "Source", :foreign_key => "source"
	belongs_to :src_file, :class_name => "PkgFile", :foreign_key => "file"
end

class PkgFile < ActiveRecord::Base
	set_table_name "files"

	# can bepart of  a src *or* binary package
	# needs more complex finders
	#belongs_to :src_file_asso, :class_name => "SrcFileAsso", :foreign_key => "file"
	#has_many :souce_pkgs, :source => "src_pkg", :through => :src_file_asso
end

