
require 'xmpp4r'
require 'xmpp4r/version/iq/version'
require 'xmpp4r/version/helper/simpleresponder'
require 'xmpp4r/muc'
require 'shellwords'
require 'escape'


class PfBot
  PRODUCT = "Proformatique Bot"
  VERSION = "0.1"

  MAX_WAIT_TIME = 10

  def initialize(jidstr, passwd, channel_jid, channel_passwd)
    @jid = Jabber::JID.new(jidstr)
    @passwd = passwd
    @channel_jid = channel_jid
    @channel_passwd = channel_passwd

    @away_msg = nil

    @cl = Jabber::Client.new(@jid)
    @cl.allow_tls = false   # hardcoded at the moment

    reconnect
    @cl.on_exception { sleep 5; reconnect }

    @commands_mutex = Mutex.new
    @sent_commands = {}
    @sent_command_number = 0

    @timer = nil
    init_timer
  end

  def reconnect
    @cl.connect # use SRV records or DIE !!!
    @cl.auth(@passwd)

    away(@away_msg) if @away_msg

    Jabber::Version::SimpleResponder.new(@cl, PRODUCT, VERSION, "")

    @muc = Jabber::MUC::SimpleMUCClient.new(@cl)
    @muc_nick = @jid.node.capitalize

    @muc.on_message {|time, nick, text|
      process_message(time, nick, text)
    }

    @muc.join("#{@channel_jid}/#{@muc_nick}", @channel_passwd)
  end

  def away(msg = nil)
    if msg.nil?
      @cl.send(Jabber::Presence.new.set_show(nil))
    else
      @cl.send(Jabber::Presence.new.set_show(:xa).set_status(msg))
    end
    @away_msg = msg
  end

  def send_command(cmdline, &callback)
    id = generate_command_id()
    @commands_mutex.synchronize {
      @sent_commands[id] = {
        :cmdline => cmdline,
        :callback => callback,
        :sent_time => Time.now.to_i,
        :replied => false
      }
    }
    @muc.say "REQUEST #" + id + " " + cmdline
  end

  def quit()
    @timer.kill if @timer
    @muc.exit
    @cl.close
  end

  def chan_members
    @muc.roster.keys.sort
  end

  def reply_failed(id, reason = nil)
    msg = id + " FAILED"
    msg += " " + Escape.shell_command(reason).to_s if reason
    @muc.say msg
  end

  def reply_done(id)
    @muc.say id + " DONE"
  end

  def reply_conditional(id, reason = nil)
    if yield
      reply_done(id)
    else
      reply_failed(id, reason)
    end
  end

  def reply_result(id, res)
    @muc.say id + " RESULT: " + res
  end

  def announce(msg)
    @muc.say "ANNOUNCE " + Escape.shell_command(msg).to_s
  end

  private

  def unnormalize_message(msg)
    REXML::DocType::DEFAULT_ENTITIES.each_value do |entity|
      msg = msg.gsub("&#{entity.name};", entity.value)
    end
    msg = msg.gsub( '&amp;', '&' )
    msg
  end

  def process_message(time, nick, text)
    cmdline = Shellwords.shellwords(unnormalize_message(text))
    if cmdline.first[-1..-1] == ":"
      return if time    # ignore backlog

      order_to = cmdline.first[0..-2].downcase

      # only execute if order is for us
      execute_order(nick, cmdline) if order_to == "all" or order_to == @muc_nick.downcase

    elsif cmdline.first[0..0] == "#"
      return if time    # ignore backlog

      process_command_reply(nick, cmdline)

    elsif cmdline.first == "REQUEST"
      return if time    # ignore backlog

      cmdline.shift
      id = cmdline.shift
      if id.nil? or id[0..0] != "#" or cmdline.size < 2
        @muc.say "malformed request", nick
      end

      subsys = cmdline.shift.upcase
      process_command(id, subsys, cmdline)

    else
      process_unknown(time, nick, cmdline)
    end
  end

  def execute_order(nick, cmdline)
    cmd = cmdline.shift.downcase
  
    # TODO: check if nick is authorized
  
    case cmd
    when "die":
      quit
    else
      @muc.say "unknown command", nick
    end
  end

  def process_command_reply(nick, cmdline)
    reply_id = cmdline.shift[1..-1]
    result = cmdline.shift
    cb = nil

    @commands_mutex.synchronize {
      if @sent_commands.has_key?(reply_id)
        cb = @sent_commands[reply_id][:callback]
  
        # reset timeout timer
        @sent_commands[reply_id][:sent_time] = Time.now.to_i

        # check if finished
        @sent_commands[reply_id][:replied] = true if ["DONE", "FAILED"].include?(result)
      end
    }

    if cb
      case result
      when "DONE", "FAILED"
        extra = cmdline.shift
      when "RESULT:"
        extra = cmdline.join(" ")
      else
        return
      end

      cb.call(nick, result, extra)
    end
  end

  def process_command(id, subsys, cmdline)
    # Hey ! Do something, lazy bot !
  end

  def generate_command_id()
    id = nil
    @commands_mutex.synchronize {
      id = @muc_nick + "-" + @sent_command_number.to_s.rjust(5, "0")
      @sent_command_number += 1
    }
    return id
  end

  def process_unknown(time, nick, cmdline)
    # Do nothing by default
  end

  def init_timer()
    @timer = Thread.new {
      while (true)
        sleep(1)
        @sent_commands.each_pair do |id, data|
          if (Time.now.to_i - data[:sent_time]) > MAX_WAIT_TIME
            @muc.say id + " COMMAND '#{data[:cmdline]}' TIMED OUT" if not data[:replied]
            cb = data[:callback]
            cb.call(nil, "TIMEOUT", nil)
            @sent_commands.delete(id)
          end
        end
      end
    }
  end

end

